<?php

/**
 * Class ExpressionTest
 *
 * Date: 28.03.13
 * Time: 15:51
 * @author Thomas Joußen <tjoussen@databay.de>
 */
class ExpressionTest extends PHPUnit_Framework_TestCase {


	public function test_AnswerOfQuestionExpression()
	{
		include_once __DIR__ . '/../../classes/Expressions/AnswerOfQuestionExpression.php';
		$expression = new AnswerOfQuestionExpression();
		$expression->parseValue("Q3672");

		$this->assertEquals("Q3672", $expression->getValue());
		$this->assertEquals(3672, $expression->getQuestionIndex());
	}

	public function test_NumberOfResultExpression()
	{
		include_once __DIR__ . '/../../classes/Expressions/NumberOfResultExpression.php';
		$expression = new NumberOfResultExpression();
		$expression->parseValue("+34+");

		$this->assertEquals("+34+", $expression->getValue());
		$this->assertEquals(34, $expression->getNumericValue());
	}

	public function test_NumericResultExpression()
	{
		include_once __DIR__ . '/../../classes/Expressions/NumericResultExpression.php';
		$expression = new NumericResultExpression();
		$expression->parseValue("#-4.5#");

		$this->assertEquals("#-4.5#", $expression->getValue());
		$this->assertEquals(-4.5, $expression->getNumericValue());
	}

	public function test_PercentageResultExpression()
	{
		include_once __DIR__ . '/../../classes/Expressions/PercentageResultExpression.php';
		$expression = new PercentageResultExpression();
		$expression->parseValue("%99.9%");

		$this->assertEquals("%99.9%", $expression->getValue());
		$this->assertEquals(99.9, $expression->getNumericValue());
	}

	public function test_ResultOfAnswerOfQuestionExpression()
	{
		include_once __DIR__ . '/../../classes/Expressions/ResultOfAnswerOfQuestionExpression.php';
		$expression = new ResultOfAnswerOfQuestionExpression();
		$expression->parseValue("Q36[49]");

		$this->assertEquals("Q36[49]", $expression->getValue());
		$this->assertEquals(36, $expression->getQuestionIndex());
		$this->assertEquals(49, $expression->getAnswerIndex());
	}

	public function test_StringResultExpression()
	{
		include_once __DIR__ . '/../../classes/Expressions/StringResultExpression.php';
		$expression = new StringResultExpression();
		$expression->parseValue("~Hallo Welt~");

		$this->assertEquals("~Hallo Welt~", $expression->getValue());
		$this->assertEquals("Hallo Welt", $expression->getText());
	}

	public function test_MatchingResultExpression()
	{
		include_once __DIR__ . '/../../classes/Expressions/MatchingResultExpression.php';
		$expression = new MatchingResultExpression();
		$expression->parseValue(";14:15;");

		$this->assertEquals(";14:15;", $expression->getValue());
		$this->assertEquals(14, $expression->getLeftNumericValue());
		$this->assertEquals(15, $expression->getRightNumericValue());
	}

	public function test_OrderingResultExpression()
	{
		include_once __DIR__ . '/../../classes/Expressions/OrderingResultExpression.php';
		$expression = new OrderingResultExpression();

		$expression->parseValue("$14$");
		$this->assertEquals("$14$", $expression->getValue());
		$this->assertEquals(array(14), $expression->getOrdering());

		$expression->parseValue("$14,15$");
		$this->assertEquals("$14,15$", $expression->getValue());
		$this->assertEquals(array(14,15), $expression->getOrdering());

		$expression->parseValue("$14,15,17,18,29,30,30$");
		$this->assertEquals("$14,15,17,18,29,30,30$", $expression->getValue());
		$this->assertEquals(array(14,15,17,18,29,30,30), $expression->getOrdering());
	}
}
