<?php

require_once "abstract.ilPluginModel.php";

/**
 * Class ilQuestionSetPoolPath
 *
 * Date: 21.11.13
 * Time: 11:02
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilQuestionSetPoolPath extends ilPluginModel
{

	/**
	 * @var integer
	 */
	protected $id;

	/**
	 * @var string
	 */
	protected $title;

	/**
	 * @var ilQuestionSetPoolNodeList
	 */
	protected $nodes;

	/**
	 * @var ilQuestionSetPoolSet
	 */
	protected $set;

	/**
	 * @var integer
	 */
	private $start_question_id;

	/**
	 * @var int
	 */
	private $index;

	/**
	 * @param int $id
	 */
	public function setId($id)
	{
		$this->id = $id;
	}

	/**
	 * @return int
	 */
	public function getId()
	{
		return $this->id;
	}

	/**
	 * @param \ilQuestionSetPoolNodeList $nodes
	 */
	public function setNodes($nodes)
	{
		$this->nodes = $nodes;
	}

	/**
	 * @return \ilQuestionSetPoolNodeList
	 */
	public function getNodes()
	{
		return $this->nodes;
	}

	/**
	 * @param string $title
	 */
	public function setTitle($title)
	{
		$this->title = $title;
	}

	/**
	 * @return string
	 */
	public function getTitle()
	{
		return $this->title;
	}

	/**
	 * @param \ilQuestionSetPoolSet $set
	 */
	public function setSet($set)
	{
		$this->set = $set;
	}

	/**
	 * @return \ilQuestionSetPoolSet
	 */
	public function getSet()
	{
		return $this->set;
	}

	/**
	 * @param int $index
	 */
	public function setIndex($index)
	{
		$this->index = $index;
	}

	/**
	 * @return int
	 */
	public function getIndex()
	{
		return $this->index;
	}

	/**
	 * @param int $question_id
	 *
	 * @return bool
	 */
	public function isQuestionInPath($question_id)
	{
		$data = $this->getNodes()->getData();
		foreach($data as $item)
		{
			if($item["question_fi"] == $question_id)
			{
				return true;
			}
		}
		return false;
	}

	/**
	 * @param integer $id
	 * @return bool
	 */
	public static function exists($id)
	{
		/**
		 * @var $ilDB ilDB
		 */
		global $ilDB;

		$result = $ilDB->queryF(
			"SELECT COUNT(*) cnt FROM rep_robj_xqsp_path WHERE id = %s",
			array("integer"),
			array($id)
		);
		$data = $ilDB->fetchAssoc($result);

		return (int)$data['cnt'] == 1 ? true : false;
	}

	public function update()
	{
		/**
		 * @var ilDB $ilDB;
		 */
		global $ilDB;

		if($this->id > 0)
		{
			$ilDB->update(
				"rep_robj_xqsp_path",
				array("title" => array("text", $this->getTitle())),
				array("id" => array("integer", $this->getId()))
			);
		}
		else
		{
			$this->setId($ilDB->nextId("rep_robj_xqsp_path"));
			$success = $ilDB->queryF(
				"INSERT INTO rep_robj_xqsp_path (id, title, questionset_fi, path_index)
				(SELECT %s,%s, %s,
				CASE WHEN MAX(path_index) IS NULL
					THEN 1
					ELSE MAX(path_index) + 1
				END
				FROM rep_robj_xqsp_path WHERE questionset_fi = %s)",
				array("integer", "text", "integer", "integer"),
				array($this->getId(), $this->getTitle(), (int)$_GET['set_id'],(int)$_GET['set_id'])
			);

			$node_id = $ilDB->nextId("rep_robj_xqsp_node");
			return $success && $ilDB->insert(
				"rep_robj_xqsp_node",
				array(
					"id" => array("integer", $node_id),
					"question_fi" => array("integer", $this->start_question_id),
					"path_fi" => array("integer", $this->getId()),
					"node_index" => array("integer", 1),
					"initial_condition" => array("text", "")
				)
			);
		}
	}
	
	/**
	 * Extracted from update method for XML-Import procedure
	 * 
	 * @return bool
	 */
	public function import()
	{
		/**
		 * @var ilDB $ilDB;
		 */
		global $ilDB;
		
		$this->setId($ilDB->nextId("rep_robj_xqsp_path"));
		$success = $ilDB->queryF(
				"INSERT INTO rep_robj_xqsp_path (id, title, questionset_fi, path_index)
				(SELECT %s,%s, %s,
				CASE WHEN MAX(path_index) IS NULL
					THEN 1
					ELSE MAX(path_index) + 1
				END
				FROM rep_robj_xqsp_path WHERE questionset_fi = %s)",
				array("integer", "text", "integer", "integer"),
				array($this->getId(), $this->getTitle(), (int)$this->getSet()->getId(),(int)$this->getSet()->getId())
		);
		return $success;
	}

	public function read()
	{
		require_once "class.ilQuestionSetPoolNodeList.php";
		require_once "class.ilQuestionSetPoolSet.php";
		/** @var ilDB $ilDB */
		global $ilDB;

		$result = $ilDB->queryF(
			"SELECT * FROM rep_robj_xqsp_path WHERE id = %s",
			array("integer"),
			array($this->id)
		);

		$data = $ilDB->fetchAssoc($result);

		$this->setId($data['id']);
		$this->setTitle($data['title']);
		$this->setIndex($data["path_index"]);
		$this->setNodes(new ilQuestionSetPoolNodeList($this));

		$id = isset($data["questionset_fi"])? $data["questionset_fi"] : $_GET['set_id'];
		$set = new ilQuestionSetPoolSet($id);
		$set->read();
		$this->setSet($set);
	}

	/**
	 * @param array $question_ids
	 */
	public function duplicate($question_ids, $title = "")
	{
		/** @var ilDB $ilDB */
		global $ilDB;

		$clone = clone $this;
		$clone->setId($ilDB->nextId("rep_robj_xqsp_path"));

		if($title != "")
		{
			$clone->setTitle($title);
		}

		$ilDB->queryF(
			"INSERT INTO rep_robj_xqsp_path (id, title, questionset_fi, path_index)
			(SELECT %s,%s, %s,
			CASE WHEN MAX(path_index) IS NULL
				THEN 1
				ELSE MAX(path_index) + 1
			END
			FROM rep_robj_xqsp_path WHERE questionset_fi = %s)",
			array("integer", "text", "integer", "integer"),
			array($clone->getId(), $clone->getTitle(),$clone->getSet()->getId(),$clone->getSet()->getId())
		);

		require_once "class.ilQuestionSetPoolNode.php";
		$data = $this->getNodes()->getData();
		foreach($data as $element)
		{
			$node = new ilQuestionSetPoolNode($element["id"]);
			$node->read();
			$node->duplicate($clone, $question_ids);
		}
	}

	public function remove()
	{
		global $ilDB;

		$this->removeNodes();

		return $ilDB->manipulateF(
			"DELETE FROM rep_robj_xqsp_path WHERE id = %s",
			array("integer"),
			array($this->getId())
		);
	}

	public function getQuestionsLowerAndEqualsIndex($index)
	{
		global $ilDB;

		$result = $ilDB->queryF(
			"SELECT qs_qst.question_index, q.title, q.question_id
			FROM rep_robj_xqsp_qs_qst qs_qst
			INNER JOIN rep_robj_xqsp_node n ON n.question_fi = qs_qst.question_fi
			INNER JOIN qpl_questions q ON q.question_id = qs_qst.question_fi
			WHERE n.path_fi = %s AND n.node_index <= %s
			ORDER BY n.node_index",
			array("integer", "integer"),
			array($this->getId(), $index)
		);

		$data = array();
		while(($row = $ilDB->fetchAssoc($result)) != null)
		{
			$data[] = $row;
		}
		return $data;
	}

	/**
	 * @param ilQuestionSetPoolPath $path
	 * @param int $index
	 *
	 * @return bool
	 */
	public function isEqualsUntilIndex($path, $index)
	{
		$current_nodes = $this->getNodes()->getData();
		$compate_nodes = $path->getNodes()->getData();

		for($i = 0; $i < $index; $i++)
		{
			if(
				$current_nodes[$i]["question_index"] != $compate_nodes[$i]["question_index"]
			)
			{
				return false;
			}
		}
		return true;
	}

	/**
	 * @param ilPropertyFormGUI $form
	 */
	public function bindForm(ilPropertyFormGUI $form)
	{
		$this->setTitle($form->getInput("title"));
		$this->start_question_id = $form->getInput("start_question");
	}

	private function removeNodes()
	{
		require_once "class.ilQuestionSetPoolNode.php";

		$data = $this->getNodes()->getData();
		foreach ($data as $element)
		{
			$node = new ilQuestionSetPoolNode($element["id"]);
			$node->read();
			$node->remove();
		}
	}
}
 