<?php

require_once "abstract.ilPluginModelList.php";

/**
 * Class ilQuestionSetPoolSetList
 *
 * Date: 10.10.13
 * Time: 10:49
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilQuestionSetPoolSetList extends ilPluginModelList
{
	/**
	 * @var ilObjQuestionSetPool
	 */
	protected $question_set_pool;
	
	/**
	 * needed for formatest-importer
	 * @var array
	 */
	public $xqsp_qst_mapping = array();
	
	/**
	 * needed for formatest-importer
	 * @var array
	 */
	public $set_mapping = array();

	public function __construct($question_set_pool)
	{
		$this->question_set_pool = $question_set_pool;

		parent::__construct();
	}

	/**
	 * @param array $params
	 * @param array $filter
	 *
	 * @return array
	 */
	public function loadData($params = array(), $filter = array())
	{
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/providers/class.ilQuestionSetPoolSetsProvider.php";

		$provider = new ilQuestionSetPoolSetsProvider();
		$filter["questionsetpool_fi"] = $this->question_set_pool->getId();
		return $provider->loadData($filter, $params);
	}

	public function getSet($id)
	{
		$data = $this->getData();

		foreach($data as $item)
		{
			if($item->getId() == $id)
			{
				return $item;
			}
		}

		return null;
	}
	
	/**
	 * @param      $set_ids
	 * @param bool $do_clone
	 * @return bool
	 */
	public function insert($set_ids, $do_clone = true)
	{
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/class.ilObjQuestionSetPool.php";
		
		foreach($set_ids as $set_id)
		{
			$set = new ilQuestionSetPoolSet($set_id);
			$set->read();
			
			if($set != null )
			{
				if($do_clone)
				{
					$clone = $set->duplicate($this->question_set_pool->getId());
					$container_id = $clone->getId();
					$this->set_mapping[$set_id] = $clone->getId();
					$this->xqsp_qst_mapping = $set->getQuestionMapping();
				}
				else
				{
					$container_id = $set->getId();
					$this->xqsp_qst_mapping = $set->getQuestionMapping();
				}
				$success = $this->db->queryF(
					"INSERT INTO rep_robj_xtsf_qs (obj_fi, questionset_fi, sorting)
					(SELECT %s,%s,
					CASE WHEN MAX(sorting) IS NULL
						THEN 10
						ELSE MAX(sorting) + 10
					END
					FROM rep_robj_xtsf_qs WHERE obj_fi = %s)",
					array("integer", "integer", "integer"),
					array($this->question_set_pool->getId(), $container_id, $this->question_set_pool->getId())
				);
				
				if(!$success)
				{
					return false;
				}
			}
		}
		return true;
	}

	public function remove($question_ids)
	{
		return true;
	}

	/**
	 * @param array $data
	 *
	 * @return array
	 */
	public function createObjectsFromData($data)
	{
		$objects = array();
		foreach($data as $item)
		{
			require_once 'class.ilQuestionSetPoolSet.php';
			$set = new ilQuestionSetPoolSet($this->question_set_pool);
			$set->setId($item['id']);
			$set->setTitle($item['title']);
			$set->setDescription($item['description']);

			$objects[] = $set;
		}

		return $objects;
	}
	
	/**
	 * @return array
	 */
	public function getQuestionMapping()
	{
		return $this->xqsp_qst_mapping;
	}
	
	/**
	 * @return array
	 */
	public function getSetMapping()
	{
		return $this->set_mapping;
	}
}
