<?php

require_once 'class.ilPluginTableGUI.php';
require_once 'Services/UIComponent/AdvancedSelectionList/classes/class.ilAdvancedSelectionListGUI.php';
require_once 'Services/Form/classes/class.ilNumberInputGUI.php';

/**
 * Class ilQuestionSetPoolNodeTableGUI
 *
 * Date: 21.11.13
 * Time: 11:59
 * @author Thomas Joußen <tjoussen@databay.de>
 */ 
class ilQuestionSetPoolNodeTableGUI extends ilPluginTableGUI
{

	private $controller;

	public function __construct($controller, $plugin, $path_id)
	{
		global $ilCtrl;

		$this->ctrl = $ilCtrl;
		$this->controller = $controller;
		$this->setId("xqsp_nodes_".$plugin->getId() . '_' . $path_id);

		parent::__construct($controller, "ilQuestionSetPoolNodesGUI.listNodes", $plugin);

		$this->createTable();
	}

	private function createTable()
	{
		$this->setTitle($this->plugin->txt('nodes'));

		$this->setFormName("paths");
		$this->setStyle('table', 'fullwidth');

		$this->addColumn($this->plugin->txt('index'), '', true, '1%');
		$this->addColumn($this->lng->txt('question'), '', '400px');
		$this->addColumn($this->plugin->txt('initial_condition'));
		$this->addColumn($this->plugin->txt('jump_conditions'));
		$this->addColumn($this->plugin->txt('dominant_scoring'));
		$this->addColumn($this->plugin->txt('actions'), '', '1%');

		/*$this->setSelectAllCheckbox('n_id');*/

		$this->setRowTemplate('tpl.il_xqsp_node_row.html', $this->controller->plugin->getDirectory());
		$this->ctrl->setParameter($this->controller, "path_id", $_GET['path_id']);
		$this->ctrl->setParameter($this->controller, "set_id", $_GET['set_id']);
		$this->setFormAction($this->ctrl->getFormAction($this->controller));
		$this->ctrl->setParameter($this->controller, "path_id", "");
		$this->ctrl->setParameter($this->controller, "set_id", "");
		$this->setDefaultOrderField("title");
		$this->setDefaultOrderDirection("asc");

		$this->enable("sort");
		$this->enable("header");
		/*$this->enable("select_all");*/
	}


	/**
	 * Return an array of all static (always visible) data fields in a row.
	 * For each key there has to be a variable name VAL_<COLUMN_KEY> in your defined row template.
	 * Example:
	 *     return array('title', 'checkbox');
	 *     There have to be two template variables: VAL_TITLE and VAL_CHECKBOX
	 * @return array
	 */
	protected function getStaticData()
	{
		return array(
			"node_index",
			"question_title",
			"question_index",
			"initial_condition",
			"jump_conditions",
			"dominant_scoring",
			"actions"
		);
	}

	protected function formatCellValue($column, array $row)
	{
		switch($column)
		{
			case "question_title":
				$this->ctrl->setParameter($this->parent_obj, "q_id", $row["question_fi"]);
				$this->ctrl->setParameter($this->parent_obj, "set_id", $row["set_fi"]);
				$this->ctrl->setParameter($this->parent_obj, "path_id", $row["path_fi"]);
				$row['question_title'] = '<a href="' . $this->ctrl->getLinkTarget($this->parent_obj, "ilQuestionSetPoolNodesGUI.editQuestion") . '" title="' . $this->plugin->txt('edit_question') . '">' . $row['question_title'] . '</a>';
				$this->ctrl->setParameter($this->parent_obj, "q_id", "");
				$this->ctrl->setParameter($this->parent_obj, "set_id", "");
				$this->ctrl->setParameter($this->parent_obj, "path_id", "");
				break;
			case "node_index":
				$sort = new ilNumberInputGUI('', 'sorting[' . $row['id'] . ']');
				$sort->allowDecimals(false);
				$sort->setSize(3);
				$sort->setValue($row[$column]);
				if($row[$column] == 1)
				{
					$sort->setDisabled(true);
				}
				$row[$column] = $sort->render();
				break;
			case "dominant_scoring":
				$number = new ilNumberInputGUI("", $column."[".$row['id']. "]");
				$number->allowDecimals(false);
				$number->setSize(3);
				$number->setValue($row[$column]);
				$row[$column] = $number->render();
				break;
			case "initial_condition":
				$row = $this->renderInitialCondition($row);
				break;
			case "actions":
				$action = new ilAdvancedSelectionListGUI();
				$action->setId('asl_'.$row["id"]);
				$action->setListTitle($this->lng->txt('actions'));

				$this->ctrl->setParameter($this->controller, "set_id", $row["set_fi"]);
				$this->ctrl->setParameter($this->controller, "path_id", $row["path_fi"]);

				$this->ctrl->setParameter($this->controller, "node_id", $row['id']);
				if(!$row["initial_condition"])
				{
					
					$action->addItem($this->plugin->txt("add_initial_condition"), '', $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolInitialConditionGUI.edit"));
				}
				else
				{
					$action->addItem($this->plugin->txt("edit_initial_condition"), '', $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolInitialConditionGUI.edit"));
				}
				$this->ctrl->setParameter($this->controller, "node_id", "");

				$this->ctrl->setParameter($this->controller, "node_fi", $row['id']);
				$this->ctrl->setParameter($this->controller, "condition_id", $row['condition_id']);
				$action->addItem($this->plugin->txt("jump_condition_add"), '', $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolJumpConditionGUI.edit"));
				$this->ctrl->setParameter($this->controller, "condition_id", "");
				$this->ctrl->setParameter($this->controller, "node_fi", "");

				$this->ctrl->setParameter($this->controller, 'node_id', $row['id']);
				$action->addItem($this->lng->txt("delete"), '',$this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolNodeGUI.removeConfirmation"));
				$this->ctrl->setParameter($this->controller, "node_id", "");

				$this->ctrl->setParameter($this->controller, "set_id", '');
				$this->ctrl->setParameter($this->controller, "path_id", '');

				$row["actions"] = $action->getHtml();
				break;
			case "jump_conditions":
				$row = $this->renderJumpConditions($row);
				break;
		}
		return parent::formatCellValue($column, $row);
	}

	/**
	 * @param array $row
	 *
	 * @return array
	 */
	protected function renderJumpConditions(array $row)
	{
		require_once "./Customizing/global/plugins/Services/Repository/RepositoryObject/QuestionSetPool/classes/models/class.ilQuestionSetPoolNode.php";

		$model = new ilQuestionSetPoolNode($row["id"]);
		$model->read();
		$row["jump_conditions"] = $model->getJumpConditionList()->getData();

		$this->ctrl->setParameter($this->controller, "set_id", $row["set_fi"]);
		$this->ctrl->setParameter($this->controller, "path_id", $row["path_fi"]);

		$conditions = '<ul style="padding: 0; margin:0;">';
		foreach ($row['jump_conditions'] as $key => $condition)
		{
			$this->ctrl->setParameter($this->controller, "condition_id", $condition['id']);
			$edit = $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolJumpConditionGUI.edit");
			$remove = $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolJumpConditionGUI.remove");
			$up = $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolJumpConditionGUI.up");
			$down = $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolJumpConditionGUI.down");
			$this->ctrl->setParameter($this->controller, "condition_id", "");

			$conditions .= "<li>";
			if($key != 0)
			{
				$conditions .= '<a class="ilQuestionSetPoolConditionUp" href="'.$up.'">&nbsp;</a>';
			}
			else
			{
				$conditions .= '<span class="ilQuestionSetPoolConditionButtonPlaceholder">&nbsp;</span>';
			}

			$conditions .= $condition['jump_condition'] . " - ";
			if ($condition["question_index"] != null)
			{
				$conditions .= $this->plugin->txt("target_question") . ": ";
				$conditions .= "Q" . $condition["question_index"];
			}
			else
			{
				$conditions .= $this->plugin->txt("tst_termination");
			}

			$action = new ilAdvancedSelectionListGUI();
			$action->setId('asl_jump_'.$condition["id"]);
			$action->setListTitle('');
			$action->addItem($this->plugin->txt("jump_condition_edit"), '', $edit);
			$action->addItem($this->plugin->txt("jump_condition_delete"), '', $remove);

			$conditions .= $action->getHTML();
			$conditions .= "<br />";

			if($key != count($row["jump_conditions"])-1)
			{
				$conditions .= '<a class="ilQuestionSetPoolConditionDown" href="'.$down.'">&nbsp</a>';
			}
			else
			{
				$conditions .= '<span class="ilQuestionSetPoolConditionButtonPlaceholder">&nbsp;</span>';
			}

			if($condition["repititions"] == null)
			{
				$condition["repititions"] = $this->plugin->txt("endless_repititions");
			}

			$conditions .= $this->plugin->txt("repititions") . ": " . $condition["repititions"];
			$conditions .= "</li>";
		}
		if (count($row['jump_conditions']))
		{
			$row['jump_conditions'] = $conditions;
		}
		else
		{
			$row['jump_conditions'] = "";
		}

		$this->ctrl->setParameter($this->controller, "set_id", '');
		$this->ctrl->setParameter($this->controller, "path_id", '');

		return $row;
}

	/**
	 * @param array $row
	 * @return array
	 */
	protected function renderInitialCondition(array $row)
	{
		if($row["initial_condition"] != null)
		{
			$this->ctrl->setParameter($this->controller, "set_id", $row["set_fi"]);
			$this->ctrl->setParameter($this->controller, "path_id", $row["path_fi"]);

			$this->ctrl->setParameter($this->controller, "node_id", $row['id']);
			$edit = $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolInitialConditionGUI.edit");
			$this->ctrl->setParameter($this->controller, "node_id", "");

			$this->ctrl->setParameter($this->controller, "node_id", $row['id']);
			$remove = $this->ctrl->getLinkTarget($this->controller, "ilQuestionSetPoolInitialConditionGUI.remove");
			$this->ctrl->setParameter($this->controller, "node_id", "");

			$action = new ilAdvancedSelectionListGUI();
			$action->setId('asl_init_'.$row["id"]);
			$action->setListTitle('');

			$action->addItem($this->plugin->txt("edit_initial_condition"), '', $edit);
			$action->addItem($this->plugin->txt("remove_initial_condition"), '', $remove);

			$row["initial_condition"] = '<ul style="margin:0;padding:0"><li>' . $row["initial_condition"]  . $action->getHTML().'</li></ul>';

			$this->ctrl->setParameter($this->controller, "set_id", '');
			$this->ctrl->setParameter($this->controller, "path_id", '');

			return $row;
		}

		return $row;
}
}
 